## Formula Viewer
## v 1.2 BETA
##
## 2012 - May - 23
## Added Save/Load/Delete preset functionality.
## 
## Author: A. Roslovtsev
## Email: andrei[AT]bytes-and-pixels[DOT]com
## Web: www.bytes-and-pixels.com
## 
## This tool is a helper for Vector Path plug-in for Gimp
## It allows to preview your curve interactively
## 
## This script is free software under GPL license.

import sys, csv
from os import path
from Tkinter import *
import tkMessageBox as msg
from math    import *

class fview():

    def __init__(self,Width=400, Height=400):
            #---------------------------------------------------
            # Load preset data if exists, else create empty file
            def open_presets():
                self.presets = []
                if path.isfile(sys.path[0]+'/formula_presets.csv')!= True:
                    f = open(sys.path[0]+'/formula_presets.csv','w')
                    f.close

                f =  open(sys.path[0]+'/formula_presets.csv','r')
                data = csv.reader(f, delimiter=',',quotechar='"')
                for datum in data:
                    self.presets.append(datum)

                f.close()
                
            def save_presets():
                f =  open(sys.path[0]+'/formula_presets.csv','wb')
                data = csv.writer(f, delimiter=',',quotechar='"')
                self.presets=sorted(self.presets,key=lambda x: str.lower(x[0]))
                for preset in self.presets:
                    data.writerow(preset)

                f.close()
                
            def update_list():
                 self.items=[]
                 for preset in self.presets:
                     self.items.append(preset[0])
            
                 self.items = sorted(self.items,key=str.lower)
                 #populate list box with names
                 pres_list.delete(0, END)
                 for item in self.items:
                     pres_list.insert(END, item)

                 pres_list.select_set(0)

            def load_preset():
                if len(pres_list.curselection())>0:
                    
                    presets=self.presets

                    for preset in self.presets:
                        if preset[0] == pres_list.get(pres_list.curselection()):
                            name_fld.delete(0,END)
                            name_fld.insert(0,preset[0])
                            x_fld.delete(0,END)
                            x_fld.insert(0,preset[1])
                            y_fld.delete(0,END)
                            y_fld.insert(0,preset[2])

                            update(self)
    
            def find(f, seq): 
               """Return first item in sequence where f(item) == True."""
               for item in seq:
                   if f==item:
                       return True
                       
                    
            def save_preset():
                exists = find(name_fld.get(), self.items)
                if exists:
                    msg.showwarning("Name conflict","Preset name already in use.")

                else:
                    presets = self.presets
                    new = []
                    new.append(name_fld.get())           
                    new.append(x_fld.get())
                    new.append(y_fld.get())
                    presets.append(new)

                    save_presets()
                    update_list()
            
            def delete_preset():
                name = pres_list.get(pres_list.curselection())
                presets = self.presets
                index=0
                for preset in presets:
                    if preset[0]==name:
                        del presets[index]

                    index = index+1

                save_presets()
                update_list()

            # Callbacks
            def update(event):
                points = int(p_fld.get())
                xf = x_fld.get()
                yf = y_fld.get()
                
                self.process(xf,yf,points,canvas)

            def select_active(event):
                pres_list.select_clear(pres_list.curselection())
                pres_list.select_set(ACTIVE)
                load_preset()


            def scroll_to_letter(event):
                index=0

                for item in self.items:
                    
                    if(item[0:1].lower()== event.char.lower()):
                        pres_list.select_clear(pres_list.curselection())
                        pres_list.activate(index)
                        pres_list.select_set(index)
                        pres_list.see(index)
                        break

                    index=index+1

            #store presets in a list
            self.presets=[]
            open_presets()
            self.items=[]
            
            # Create GUI window ------------------------------
            
            win = Tk()
            win.title ("Formula Viewer v1.2 (with presets)")

            main_frame = Frame()
            name_lbl = Label(main_frame,text="Name:")
            name_lbl.grid(row=0)

            name_fld = Entry(main_frame,width=20)
            name_fld.insert(0,"Untitled")
            name_fld.grid(row=0, column=1,sticky=W)
            
            x_lbl = Label(main_frame,text="X:")
            x_lbl.grid(row=1)

            x_fld = Entry(main_frame,width=100)
            x_fld.bind("<Return>",update)
            x_fld.insert(1,"(i*0.8)*10")
            x_fld.grid(row=1, column=1)

            y_lbl = Label(main_frame,text="Y:")
            y_lbl.grid(row=2)
            
            y_fld = Entry(main_frame,width=100)
            y_fld.insert(0,"200+(sin(i*0.2)*20)")
            y_fld.bind("<Return>",update)
            y_fld.grid(row=2, column=1)

            p_lbl = Label(main_frame,text="Points:")
            p_lbl.grid(row=3)
            
            p_fld = Spinbox(main_frame,width=10,from_=10, to=10000)
            p_fld.delete(0,END)
            p_fld.insert(0,100)
            p_fld.bind("<Return>",update)
            p_fld.grid(row=3,column=1,sticky=W)
            
            canvas = Canvas(main_frame,width=Width,height=Height)
            canvas.create_line(0,Height/2,Width,Height/2)
            canvas.grid(row=4, columnspan=3)

            note_lbl = Label(main_frame,text="Use this tool to create your curve's formulae (hit ENTER to refresh), then paste to the plugin dialog fields.")
            note_lbl.grid(row=5,columnspan=3)

            main_frame.pack()        

            # Separator
            s = Frame(win, relief=SUNKEN,height=1,bg="black")
            s.pack(fill=X, expand=True)

            # Presets area
            pres_frame = Frame(win, relief=RAISED)
            
            # Label
            pres_lbl = Label(pres_frame,text="Presets:")
            pres_lbl.grid(row=0)

            #Dropdown frame
            list_frame = Frame(pres_frame)
            # Srollbar
            pres_scroll = Scrollbar(list_frame)
            pres_scroll.pack(side=RIGHT, fill=Y)
            #Preset List Box
            pres_list = Listbox(list_frame, selectmode=SINGLE)
            # List of preset names

            update_list()
            pres_list.pack()
            
            #bind scrollbar to the list box
            pres_scroll.config(command = pres_list.yview)
            pres_list.config(height=3,yscrollcommand=pres_scroll.set)


            pres_list.bind("<Return>",select_active)
            pres_list.bind("<KeyPress>",scroll_to_letter)
          
            list_frame.grid(row=1,column=0,padx=10, pady=10)

            #Load button
            l_btn = Button(pres_frame, text="Load",
                           command=load_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=1,padx=5, pady=5)

            #Delete button
            l_btn = Button(pres_frame, text="Delete",
                           command=delete_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=2,padx=5, pady=5)

            #Save button
            l_btn = Button(pres_frame, text="Save",
                           command=save_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=3,padx=5, pady=5)


            pres_frame.pack(fill=X)

            # End of GUI window code ------------------------------


            points = int(p_fld.get())
            xf = x_fld.get()
            yf = y_fld.get()

            self.data = [[0.0,0.0] for i in range(points)]
            self.points = points

            self.process(xf,yf,points,canvas)
            win.mainloop()
      
    def process (self,xf, yf,points,canvas,Width=400, Height=400):
        self.data = [[0.0,0.0] for i in range(points)]
        self.points = points
        for p in range(points):
            i = p
            phi = 1.618   #1.61803399
            phi_inv = 0.618    #0.6180340

            try:
                self.data[i][0]=eval(xf)
                self.data[i][1]=eval(yf)
            except:
                msg.showwarning("Error","Your formula has an error")
                break
            
            self.plotOne (canvas,Width,Height)



    def plotOne (self, canvas, Width, Height) :
            #items = canvas.find_all()
            canvas.delete(ALL)
            for i in range(len(self.data)) :
                x = self.data[i][0]
                y = self.data[i][1]
                if i : canvas.create_line(self.data[i-1][0],yprev,x,y)
                yprev = y
def run():
    inst = fview(640,480)

if __name__ == "__main__" : run()
